import numpy as np
import pandas as pd
from sklearn.svm import SVR, NuSVR
from sklearn.preprocessing import MinMaxScaler
import matplotlib.pyplot as plt
from sklearn.metrics import mean_squared_error
from sklearn.metrics import mean_absolute_error
from sklearn.metrics import r2_score
import csv


def data_split(df):
    print(df.shape)
    column=['Close','Open', 'High', 'Low', 'Volume', 'Rpos', 'Rsize']
    df2=df[column].dropna()
    train_size=int(0.8*df.shape[0])
    test_size=int(df.shape[0]-train_size)
    X1 = df2[['Close']].values
    y = df2[['Close']].values
    X1_train,y_train=X1[0:train_size],y[0:train_size]
    X1_test,y_test=X1[train_size:],y[train_size:]
    return [X1_train,y_train,X1_test,y_test,train_size,test_size,y]


def model_SVR(X_train,y_train,X_test,y_test):
    #auto=1/(X_train.shape[1])
    regr=NuSVR(nu=0.1,kernel='rbf',gamma=0.01)
    #regr=SVR(kernel='rbf',gamma='auto')
    regr.fit(X_train,y_train)
    print('Score: %.2f' % regr.score(X_train,y_train))
    print('Score: %.2f' % regr.score(X_test, y_test))
    return regr



def mape(y_true, y_pred):
    return np.mean(np.abs(( y_true- y_pred) / y_true)) * 100




if __name__ == '__main__':
    data1 = pd.read_csv('C:/Users/Big Data/Desktop/output/实验2/fuzzy2/IXIC.csv', parse_dates=['Date'], index_col=0,
                        encoding='utf-8')
    df = data1.reset_index()
    df = df.dropna()
    seq_len=11
    X1_train,y_train,X1_test,y_test,train_size,test_size,y= data_split(df)
    scaler = MinMaxScaler(feature_range=(0, 1))
    X1_train = scaler.fit_transform(X1_train)
    a = scaler.fit(y_train)
    y_train = scaler.fit_transform(y_train)
    X1_test = scaler.fit_transform(X1_test)
    b = scaler.fit(y_test)
    y_test = scaler.fit_transform(y_test)

    # 结果比较
    X_train1, y = [], []
    for i in range(seq_len, X1_train.shape[0]):
        X_train1.append(X1_train[i - seq_len:i, :])
        y.append(y_train[i, 0])
    X1_train, y_train = np.array(X_train1), np.array(y)
    X1_train= np.reshape(X1_train, (-1, seq_len))
    #y_train=np.reshape(y_train, (-1, 1))
    X_test1, y = [], []
    for i in range(seq_len, X1_test.shape[0]):
        X_test1.append(X1_test[i - seq_len:i, :])
        y.append(y_test[i, 0])
    X1_test, y_test = np.array(X_test1), np.array(y)
    X1_test = np.reshape(X1_test, (-1, seq_len))
    #y_test = np.reshape(y_test, (-1, 1))





    regressor1 = model_SVR(X1_train, y_train,X1_test,y_test)


    #训练集
    y_train_pred= regressor1.predict(X1_train)
    #测试集
    y_test_pred= regressor1.predict(X1_test)

    print('=============训练集===============')
    print("The R2 score on the Train set is:\t{:0.8f}".format(r2_score(y_train, y_train_pred)))
    print("The MSE on the Train set is:\t{:0.8f}".format(mean_squared_error(y_train, y_train_pred)))
    print("The MAE on the Train set is:\t{:0.8f}".format(mean_absolute_error(y_train, y_train_pred)))
    print('=============测试集===============')
    print("The R2 score on the Test set is:\t{:0.8f}".format(r2_score(y_test, y_test_pred)))
    print("The MSE on the Test set is:\t{:0.8f}".format(mean_squared_error(y_test, y_test_pred)))
    print("The MAE on the Test set is:\t{:0.8f}".format(mean_absolute_error(y_test, y_test_pred)))




    #逆标准化之后的结果

    y_train = y_train.reshape(-1, 1)
    y_train_pred = y_train_pred.reshape(-1, 1)
    print(y_train.shape)
    y_train = a.inverse_transform(y_train)
    print(y_train_pred.shape)
    y_train_pred = a.inverse_transform(y_train_pred)
    # print(y_train.shape)
    # print(y_train_pred.shape)
    plt.figure(figsize=(10, 4))
    plt.plot(y_train, color='red')
    plt.plot(y_train_pred, color='blue',
             label='Predicted Price By LSTM(inverse_scale)On Train dataset')
    plt.xlabel('Time')
    plt.ylabel('Price')
    plt.legend()
    plt.show()
    rmse1 = np.sqrt(mean_squared_error(y_train, y_train_pred))
    mape1 = mape(y_train ,y_train_pred)
    print('=============训练集===============')
    print("The R2 score on the Train set is:\t{:0.8f}".format(r2_score(y_train, y_train_pred)))
    print("The MSE on the Train set is:\t{:0.8f}".format(mean_squared_error(y_train, y_train_pred)))
    print("The RMSE on the Train set is:\t{:0.8f}".format(rmse1))
    print("The MAPE on the Train set is:\t{:0.8f}".format(mape1))
    print("The MAE on the Train set is:\t{:0.8f}".format(mean_absolute_error(y_train, y_train_pred)))

    y_test = y_test.reshape(-1, 1)
    y_test_pred = y_test_pred.reshape(-1, 1)
    y_test = b.inverse_transform(y_test)
    y_test_pred = b.inverse_transform(y_test_pred)
    plt.figure(figsize=(10, 4))
    plt.plot(y_test, color='red')
    plt.plot(y_test_pred, color='blue',
             label='Predicted Price By LSTM(inverse_scale)On Test dataset')
    plt.xlabel('Time')
    plt.ylabel('Price')
    plt.legend()
    plt.show()
    rmse2 = np.sqrt(mean_squared_error(y_test, y_test_pred))
    mape2 = mape(y_test ,y_test_pred)
    print('=============测试集===============')
    print("The R2 score on the Test set is:\t{:0.8f}".format(r2_score(y_test, y_test_pred)))
    print("The MSE on the Test set is:\t{:0.8f}".format(mean_squared_error(y_test, y_test_pred)))
    print("The RMSE on the Test set is:\t{:0.8f}".format(rmse2))
    print("The MAPE on the Train set is:\t{:0.8f}".format(mape2))
    print("The MAE on the Test set is:\t{:0.8f}".format(mean_absolute_error(y_test, y_test_pred)))

    y_var_test = y_test[1:] - y_test[:len(y_test) - 1]
    y_var_predict = y_test_pred[1:] - y_test_pred[:len(y_test_pred) - 1]
    txt = np.zeros(len(y_var_test))
    for i in range(len(y_var_test - 1)):
        txt[i] = np.sign(y_var_test[i]) == np.sign(y_var_predict[i])
    result = sum(txt) / len(txt)
    print('预测涨跌正确:', result)

    print(r2_score(y_train, y_train_pred))
    print(mean_squared_error(y_train, y_train_pred))
    print(rmse1)
    print(mape1)
    print(mean_absolute_error(y_train, y_train_pred))
    print(' ')
    print(r2_score(y_test, y_test_pred))
    print(mean_squared_error(y_test, y_test_pred))
    print(rmse2)
    print(mape2)
    print(mean_absolute_error(y_test, y_test_pred))
    print(result)

    # 1. 创建文件对象
    f = open('C:/Users/Big Data/Desktop/vSVR.csv', 'w', encoding='utf-8')
    # 2. 基于文件对象构建 csv写入对象
    csv_writer = csv.writer(f)
    # 3. 构建列表头
    csv_writer.writerow(['y_test', 'vSVR'])
    # 4. 写入csv文件内容
    y_test_m = y_test.flatten().tolist()
    y_test_pred_m = y_test_pred.flatten().tolist()
    csv_writer.writerow([y_test_m, y_test_pred_m])
    # 5. 关闭文件
    f.close()


import pandas as pd
import numpy as np
from keras.models import Sequential
from keras.layers import Dense
from keras.layers import LSTM
from keras.layers import Dropout
from keras.layers import Activation
from keras.layers.core import Dense, Activation
from sklearn.preprocessing import MinMaxScaler
import matplotlib.pyplot as plt
from sklearn.metrics import mean_squared_error
from sklearn.metrics import mean_absolute_error
from sklearn.metrics import r2_score
from sklearn.svm import SVR,NuSVR
import csv

def data_split(df):
    print(df.shape)
    column=['Close', 'Open', 'High', 'Low', 'Volume', 'Rpos', 'Rsize']
    df2=df[column].dropna()
    train_size=int(0.8*df.shape[0])
    test_size=int(df.shape[0]-train_size)
    X1 = df2[['Close']].values
    X2 = df2[['Rpos', 'Rsize']].values
    y = df2[['Close']].values
    X1_train,X2_train,y_train=X1[0:train_size],X2[0:train_size],y[0:train_size]
    X1_test,X2_test,y_test=X1[train_size:],X2[train_size:],y[train_size:]
    return [X1_train,X2_train,y_train,X1_test,X2_test,y_test,train_size,test_size,y]

def ErrorSVR(regressor2,X1_train, X2_train,y_train,seq_len):
    X1_train2, y_train2 = [], []
    for i in range(seq_len, X1_train.shape[0]):
        X1_train2.append(X1_train[i - seq_len:i, :])
        y_train2.append(y_train[i, 0])
    X1_train2, y_train2 = np.array(X1_train2), np.array(y_train2)
    # print(X1_train2.shape)
    # print(y_train2.shape)
    y_train2_pred_lstm1=regressor2.predict(X1_train2)
    a = y_train2.reshape(-1, 1)
    error = a - y_train2_pred_lstm1  # 固有的预测误差
    # scaler = MinMaxScaler(feature_range=(0, 1))
    # error = scaler.fit_transform(error)
    # print(error.shape)
    X2_train2=X2_train[seq_len:,]          #选择因子

    # 根据误差和模糊输出，建立SVR模型
    regressor2 = NuSVR(nu=0.1,kernel='rbf', gamma=0.001)
    #regr=SVR(kernel='rbf',gamma='auto')
    regressor2.fit(X2_train2, error)
    return regressor2

def LSTM_model2(layers,X1_train, y_train,seq_len):  #100%历史数据
    X_train3, y_train3 = [], []
    for i in range(seq_len, X1_train.shape[0]):
        X_train3.append(X1_train[i - seq_len:i, :])
        y_train3.append(y_train[i, 0])
    X_train3, y_train3 = np.array(X_train3), np.array(y_train3)
    # Stack LSTM1
    regressor3 = Sequential()
    # Adding the first LSTM layer and some Dropout regularisation
    regressor3.add(LSTM(units=layers[0], input_shape=(X_train3.shape[1], X_train3.shape[2]), kernel_initializer='zeros',
                        return_sequences=False))  # ,activation='tanh' ,kernel_initializer='zeros',
    regressor3.add(Dropout(0.2))
    regressor3.add(Dense(units=layers[1]))
    regressor3.add(Activation("linear"))
    # Compiling the RNN
    regressor3.compile(optimizer='adam', loss='mean_squared_error')
    # es =EarlyStopping(monitor="val_loss", mode="min", patience=20, verbose=1,min_delta=0.0001)
    print(regressor3.summary())
    # Fitting the RNN to the Training set
    regressor3.fit(X_train3, y_train3, batch_size=32, epochs=100, validation_split=0.2, verbose=0)#,callbacks=[es]
    return regressor3

def mape(y_true, y_pred):
    return np.mean(np.abs(( y_true- y_pred) / y_true)) * 100


if __name__ == '__main__':
    data1 = pd.read_csv('C:/Users/Big Data/Desktop/output/实验2/fuzzy2/IXIC.csv', parse_dates=['Date'], index_col=0,
                        encoding='utf-8')
    df = data1.reset_index()
    df = df.dropna()
    seq_len=11
    X1_train,X2_train,y_train,X1_test,X2_test,y_test,train_size,test_size,y= data_split(df)
    scaler = MinMaxScaler(feature_range=(0, 1))
    X1_train = scaler.fit_transform(X1_train)
    X2_train = scaler.fit_transform(X2_train)
    a=scaler.fit(y_train)
    y_train = scaler.fit_transform(y_train)
    X1_test = scaler.fit_transform(X1_test)
    X2_test = scaler.fit_transform(X2_test)
    b=scaler.fit(y_test)
    y_test = scaler.fit_transform(y_test)
    regressor3 = LSTM_model2([38, 1], X1_train, y_train, seq_len)
    plt.plot(regressor3.history.history['loss'], label='train')
    plt.plot(regressor3.history.history['val_loss'], label='val')
    plt.legend()
    plt.show()
    regressor2=ErrorSVR(regressor3,X1_train, X2_train,y_train,seq_len)


    #结果比较
    X_train1, y = [],  []
    for i in range(seq_len, X1_train.shape[0]):
        X_train1.append(X1_train[i - seq_len:i, :])

        y.append(y_train[i, 0])
    X1_train,y_train = np.array(X_train1),  np.array(y)

    X_test1, y = [], []
    for i in range(seq_len, X1_test.shape[0]):
        X_test1.append(X1_test[i - seq_len:i, :])

        y.append(y_test[i, 0])
    X1_test,  y_test = np.array(X_test1),  np.array(y)

    #训练集
    y_train_pred_lstm2 = regressor3.predict(X1_train)
    error1= regressor2.predict(X2_train)
    print('Score: %.2f' % regressor2.score(X2_train, error1))
    error1 = error1.reshape(-1, 1)
    y_train_pred = y_train_pred_lstm2+error1[seq_len:]
    #测试集
    y_test_pred_lstm2 = regressor3.predict(X1_test)
    error2 = regressor2.predict(X2_test)
    print('Score: %.2f' % regressor2.score(X2_test, error2))
    error2 = error2.reshape(-1, 1)
    y_test_pred = y_test_pred_lstm2 + error2[seq_len:]
    # plt.plot(regressor2.history.history['loss'], label='train')
    # plt.plot(regressor2.history.history['val_loss'], label='val')
    # plt.legend()
    # plt.show()
    print('=============训练集===============')
    print("The R2 score on the Train set is:\t{:0.8f}".format(r2_score(y_train, y_train_pred)))
    print("The MSE on the Train set is:\t{:0.8f}".format(mean_squared_error(y_train, y_train_pred)))
    print("The MAE on the Train set is:\t{:0.8f}".format(mean_absolute_error(y_train, y_train_pred)))
    print('=============测试集===============')
    print("The R2 score on the Test set is:\t{:0.8f}".format(r2_score(y_test, y_test_pred)))
    print("The MSE on the Test set is:\t{:0.8f}".format(mean_squared_error(y_test, y_test_pred)))
    print("The MAE on the Test set is:\t{:0.8f}".format(mean_absolute_error(y_test, y_test_pred)))

    #逆标准化之后的结果
    y_train = y_train.reshape(-1, 1)
    y_train = a.inverse_transform(y_train)
    y_train_pred = a.inverse_transform(y_train_pred)
    # print(y_train.shape)
    # print(y_train_pred.shape)
    plt.figure(figsize=(5, 3))
    plt.plot(y_train, color='red')
    plt.plot(y_train_pred, color='blue',
             label='Predicted Price By LSTM+Svr(inverse_scale)On Train dataset')
    plt.xlabel('Time')
    plt.ylabel('Price')
    plt.legend()
    plt.show()
    rmse1 = np.sqrt(mean_squared_error(y_train, y_train_pred))
    mape1=mape(y_train,y_train_pred)
    print('=============训练集===============')
    print("The R2 score on the Train set is:\t{:0.8f}".format(r2_score(y_train, y_train_pred)))
    print("The MSE on the Train set is:\t{:0.8f}".format(mean_squared_error(y_train, y_train_pred)))
    print("The RMSE on the Train set is:\t{:0.8f}".format(rmse1))
    print("The MAPE on the Train set is:\t{:0.8f}".format(mape1))
    print("The MAE on the Train set is:\t{:0.8f}".format(mean_absolute_error(y_train, y_train_pred)))

    y_test = y_test.reshape(-1, 1)
    y_test = b.inverse_transform(y_test)
    y_test_pred = b.inverse_transform(y_test_pred)


    # plt.figure(figsize=(5, 3))
    # plt.plot(y_test, color='red')
    # plt.plot(y_test_pred, color='blue',
    #          label='Predicted Price By LSTM+Svr(inverse_scale)On Test dataset')
    # plt.xlabel('Time')
    # plt.ylabel('Price')
    # plt.legend()
    # plt.show()

    # 设置输出的图片大小
    figsize = 6, 4
    figure, ax = plt.subplots(figsize=figsize)

    # 在同一幅图片上画两条折线
    A, = plt.plot(y_test, 'r', label='actual value', linewidth=1.0)
    B, = plt.plot(y_test_pred, 'b', label='predicted value', linewidth=1.0)

    # 设置图例并且设置图例的字体及大小
    font1 = {'family': 'Times New Roman',
             'weight': 'normal',
             'size': 10,
             }
    legend = plt.legend(handles=[A, B], prop=font1)

    # 设置坐标刻度值的大小以及刻度值的字体
    plt.tick_params(labelsize=11)
    labels = ax.get_xticklabels() + ax.get_yticklabels()
    [label.set_fontname('Times New Roman') for label in labels]

    # 设置横纵坐标的名称以及对应字体格式
    font2 = {'family': 'Times New Roman',
             'weight': 'normal',
             'size': 10,
             }
    plt.xlabel('Time', font2)
    plt.ylabel('Price', font2)

    # 将文件保存至文件中并且画出图
    plt.show()




    rmse2 = np.sqrt(mean_squared_error(y_test, y_test_pred))
    mape2 = mape(y_test ,y_test_pred)
    print('=============测试集===============')
    print("The R2 score on the Test set is:\t{:0.8f}".format(r2_score(y_test, y_test_pred)))
    print("The MSE on the Test set is:\t{:0.8f}".format(mean_squared_error(y_test, y_test_pred)))
    print("The RMSE on the Test set is:\t{:0.8f}".format(rmse2))
    print("The MAPE on the Train set is:\t{:0.8f}".format(mape2))
    print("The MAE on the Test set is:\t{:0.8f}".format(mean_absolute_error(y_test, y_test_pred)))

    y_var_test = y_test[1:] - y_test[:len(y_test) - 1]
    y_var_predict = y_test_pred[1:] - y_test_pred[:len(y_test_pred) - 1]
    txt = np.zeros(len(y_var_test))
    for i in range(len(y_var_test - 1)):
        txt[i] = np.sign(y_var_test[i]) == np.sign(y_var_predict[i])
    result = sum(txt) / len(txt)
    print('预测涨跌正确:', result)


    print(r2_score(y_train, y_train_pred))
    print(mean_squared_error(y_train, y_train_pred))
    print(rmse1)
    print(mape1)
    print(mean_absolute_error(y_train, y_train_pred))
    print(' ')
    print(r2_score(y_test, y_test_pred))
    print(mean_squared_error(y_test, y_test_pred))
    print(rmse2)
    print(mape2)
    print(mean_absolute_error(y_test, y_test_pred))
    print(result)


    # 1. 创建文件对象
    f = open('C:/Users/Big Data/Desktop/GLSTM-vSVR.csv', 'w', encoding='utf-8')
    # 2. 基于文件对象构建 csv写入对象
    csv_writer = csv.writer(f)
    # 3. 构建列表头
    csv_writer.writerow(['y_test','GLSTM-vSVR'])
    # 4. 写入csv文件内容
    y_test_m=y_test.flatten().tolist()
    y_test_pred_m=y_test_pred.flatten().tolist()
    csv_writer.writerow([y_test_m,y_test_pred_m])
    # 5. 关闭文件
    f.close()


import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
# plotly is an advanced visualization tool
import chart_studio.plotly as py
from plotly.offline import init_notebook_mode, iplot
init_notebook_mode(connected=True)
import plotly.graph_objs as go
import warnings
warnings.filterwarnings("ignore")
from mpl_finance import candlestick_ohlc
import matplotlib.dates as mdates
import numpy as np
import pandas as pd
import xlrd


df=pd.read_excel("G:\\学习课件\\两篇小论文\\中文小论文\\paper2\\实验2\\输出结果\\ADI.xlsx")
print(df)


# 设置输出的图片大小
figsize = 7, 4
figure, ax = plt.subplots(figsize=figsize)

x=df['Date']
# 在同一幅图片上画两条折线
A, = plt.plot(x,df['original price'], 'r^-', label='actual value', linewidth=1.25)#, linewidth=1.25

B, = plt.plot(x,df['GLSTM-vSVR'], 'b.-', label='FCGLSTM-vSVR', linewidth=1.25)
# C, = plt.plot(x,df['GLSTM'], 'c', label='GLSTM', linewidth=1.25)
# D, = plt.plot(x,df['vSVR'], 'y', label='vSVR', linewidth=1.25)
# E, = plt.plot(x,df['BPNN'], 'b', label='BPNN', linewidth=1.25)
# F, = plt.plot(x,df['GLSTM-BPNN'], 'm', label='GLSTM-BPNN', linewidth=1.25)


# 设置图例并且设置图例的字体及大小
font1 = {'family': 'Times New Roman',
         'weight': 'normal',
         'size': 12,
         }
legend = plt.legend(handles=[A,B], prop=font1)

# 设置坐标刻度值的大小以及刻度值的字体
plt.tick_params(labelsize=12)
labels = ax.get_xticklabels() + ax.get_yticklabels()
[label.set_fontname('Times New Roman') for label in labels]

# 设置横纵坐标的名称以及对应字体格式
font2 = {'family': 'Times New Roman',
         'weight': 'normal',
         'size': 12,
         }
plt.xlabel('Time', font2)
plt.ylabel('Price', font2)

# 将文件保存至文件中并且画出图
plt.show()


import statsmodels.api as sm
import statsmodels.stats.diagnostic
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
np.set_printoptions(suppress=True)
import itertools
import warnings
import statsmodels.tsa.stattools as ts
from statsmodels.tsa.seasonal import seasonal_decompose
from statsmodels.tsa.arima_model import ARIMA
#建模分析
from imp import reload
import matplotlib.pyplot as plt
from math import sqrt as sqrt
from statsmodels.graphics.tsaplots import plot_acf, plot_pacf
import statsmodels.api as sm
import pandas as pd
import numpy as np
from imp import reload
import matplotlib.pyplot as plt
from math import sqrt as sqrt
from statsmodels.graphics.tsaplots import plot_acf, plot_pacf
import statsmodels.api as sm
from sklearn.metrics import mean_squared_error
from sklearn.metrics import mean_absolute_error
from sklearn.metrics import r2_score



def data_split(df):
    print(df.shape)
    column=['Close','Open', 'High', 'Low', 'Volume', 'Rpos', 'Rsize']
    df2=df[column].dropna()
    train_size=int(0.8*df.shape[0])
    test_size=int(df.shape[0]-train_size)
    X1 = df2[['Close']].values
    train=X1[0:train_size]
    test=X1[train_size:]
    return [train,test,train_size,test_size,X1]


# 通过网格搜索对seasonal_order进行定阶
def get_ARIMA_params(train):
    p = d =q= range(0, 2)
    pdq = [(x[0], x[1], x[2]) for x in list(itertools.product(p, d, q))]
    score_aic = 1000000.0
    warnings.filterwarnings("ignore")  # specify to ignore warning messages

    for param in pdq:
        try:

            mod = ARIMA(train, order=param)
            #             mod = sm.tsa.statespace.SARIMAX(endog=train,
            #                                             exog= trainX,
            #                                             order=param,
            #                                             seasonal_order=param_seasonal,
            #                                             simple_differencing=False)
            results = mod.fit()
            print('ARIMA{} - AIC:{}'.format(param, results.aic))
            if results.aic < score_aic:
                score_aic = results.aic
                params = param, results.aic
        except ValueError:
            print('wrong parameters:', param)
            continue
    param, results.aic = params
    print('最优ARIMA{}- AIC:{}'.format(param, results.aic))
    return param

def ARIMA_model(train,param):  #100%历史数据
    regressor1 = ARIMA(train, order=(param[0], param[1], param[2]))  # ARIMA(5,1,3)模型
    results1 = regressor1.fit(disp=0)
    return results1



def mape(y_true, y_pred):
    return np.mean(np.abs(( y_true- y_pred) / y_true)) * 100

if __name__ == '__main__':
    data1 = pd.read_csv('C:/Users/Big Data/Desktop/output/实验2/fuzzy2/WTI.csv', parse_dates=['Date'], index_col=0,
                        encoding='utf-8')
    df = data1.reset_index()
    df = df.dropna()
    #seq_len=6
    train,test,train_size,test_size,X1= data_split(df)
    param=get_ARIMA_params(train)
    #param=(2, 1, 2)
    #结果比较
    regressor1 = ARIMA_model(train,param)
    print(regressor1.summary())

    print(train_size)

    #训练集
    y_train_pred= regressor1.predict(start=1,end =train_size,dynamic=False,typ='levels')#

    #测试集
    #y_test_pred= regressor1.forecast(steps=len(test))[0]

    y_test_pred= regressor1.predict(start=train_size,end=train_size+test_size-1,dynamic=False,typ='levels')
    print(y_test_pred.shape)
    plt.figure(figsize=(10, 4))
    plt.plot(train, color='red')
    plt.plot(y_train_pred, color='blue',label='Predicted Price By ARIMA On Train dataset')
    plt.xlabel('Time')
    plt.ylabel('Price')
    plt.legend()
    plt.show()
    rmse1 = np.sqrt(mean_squared_error(train, y_train_pred))
    mape1 = mape(train ,y_train_pred)
    print('=============训练集===============')
    print("The R2 score on the Train set is:\t{:0.8f}".format(r2_score(train, y_train_pred)))
    print("The MSE on the Train set is:\t{:0.8f}".format(mean_squared_error(train, y_train_pred)))
    print("The RMSE on the Train set is:\t{:0.8f}".format(rmse1))
    print("The MAPE on the Train set is:\t{:0.8f}".format(mape1))
    print("The MAE on the Train set is:\t{:0.8f}".format(mean_absolute_error(train, y_train_pred)))


    plt.figure(figsize=(10, 4))
    plt.plot(test, color='red')
    plt.plot(y_test_pred, color='blue',
             label='Predicted Price By ARIMA On Test dataset')
    plt.xlabel('Time')
    plt.ylabel('Price')
    plt.legend()
    plt.show()
    rmse2 = np.sqrt(mean_squared_error(test, y_test_pred))
    mape2 = mape(test ,y_test_pred)
    print('=============测试集===============')
    print("The R2 score on the Test set is:\t{:0.8f}".format(r2_score(test, y_test_pred)))
    print("The MSE on the Test set is:\t{:0.8f}".format(mean_squared_error(test, y_test_pred)))
    print("The RMSE on the Test set is:\t{:0.8f}".format(rmse2))
    print("The MAPE on the Train set is:\t{:0.8f}".format(mape2))
    print("The MAE on the Test set is:\t{:0.8f}".format(mean_absolute_error(test, y_test_pred)))

    y_var_test = test[1:] - test[:len(test) - 1]
    y_var_predict = y_test_pred[1:] - y_test_pred[:len(y_test_pred) - 1]
    txt = np.zeros(len(y_var_test))
    for i in range(len(y_var_test - 1)):
        txt[i] = np.sign(y_var_test[i]) == np.sign(y_var_predict[i])
    result = sum(txt) / len(txt)
    print('预测涨跌正确:', result)

    print(' ')
    print(r2_score(train, y_train_pred))
    print(mean_squared_error(train, y_train_pred))
    print(rmse1)
    print(mape1)
    print(mean_absolute_error(train, y_train_pred))
    print(' ')
    print(r2_score(test, y_test_pred))
    print(mean_squared_error(test, y_test_pred))
    print(rmse2)
    print(mape2)
    print(mean_absolute_error(test, y_test_pred))
    print(result)


import pandas as pd
import numpy as np
from keras.models import Sequential
from keras.layers import Dense
from keras.layers import LSTM
from keras.layers import Dropout
from keras.layers import Activation
from sklearn.preprocessing import MinMaxScaler
import matplotlib.pyplot as plt
from sklearn.metrics import mean_squared_error
from sklearn.metrics import mean_absolute_error
from sklearn.metrics import r2_score
import csv
#import Fuzzification
#from keras.callbacks import EarlyStopping

def data_split(df):
    print(df.shape)
    column=['Close','Open', 'High', 'Low', 'Volume', 'Rpos', 'Rsize']
    df2=df[column].dropna()
    train_size=int(0.8*df.shape[0])
    test_size=int(df.shape[0]-train_size)
    X1 = df2[['Close']].values
    y = df2[['Close']].values
    X1_train,y_train=X1[0:train_size],y[0:train_size]
    X1_test,y_test=X1[train_size:],y[train_size:]
    return [X1_train,y_train,X1_test,y_test,train_size,test_size,y]


def LSTM_model(unit,X1_train, y_train,X1_test, y_test,batch_size,epochs):  #100%历史数据
    # Stack LSTM1
    regressor1 = Sequential()
    # Adding the first LSTM layer and some Dropout regularisation
    regressor1.add(LSTM(units=unit, input_shape=(X1_train.shape[1], X1_train.shape[2]), kernel_initializer='zeros',
                        return_sequences=False))  # ,activation='tanh' ,kernel_initializer='zeros',
    regressor1.add(Dropout(0.2))
    regressor1.add(Dense(units=1))
    regressor1.add(Activation("linear"))
    # Compiling the RNN
    regressor1.compile(optimizer='adam', loss='mean_squared_error')
    # es =EarlyStopping(monitor="val_loss", mode="min", patience=20, verbose=1,min_delta=0.0001)
    print(regressor1.summary())
    # Fitting the RNN to the Training set
    regressor1.fit(X1_train, y_train, batch_size, epochs, validation_split=0.2, verbose=0)#,callbacks=[es]
    return regressor1



def mape(y_true, y_pred):
    return np.mean(np.abs(( y_true- y_pred) / y_true)) * 100




if __name__ == '__main__':
    data1 = pd.read_csv('C:/Users/Big Data/Desktop/output/实验2/fuzzy2/IXIC.csv', parse_dates=['Date'], index_col=0,
                        encoding='utf-8')
    df = data1.reset_index()
    df = df.dropna()
    seq_len=11
    X1_train,y_train,X1_test,y_test,train_size,test_size,y= data_split(df)
    scaler = MinMaxScaler(feature_range=(0, 1))
    X1_train = scaler.fit_transform(X1_train)
    a = scaler.fit(y_train)
    y_train = scaler.fit_transform(y_train)
    X1_test = scaler.fit_transform(X1_test)
    b = scaler.fit(y_test)
    y_test = scaler.fit_transform(y_test)



    #结果比较
    X_train1, y = [], []
    for i in range(seq_len, X1_train.shape[0]):
        X_train1.append(X1_train[i - seq_len:i, :])
        y.append(y_train[i, 0])
    X1_train,  y_train = np.array(X_train1), np.array(y)

    X_test1,  y = [], []
    for i in range(seq_len, X1_test.shape[0]):
        X_test1.append(X1_test[i - seq_len:i, :])
        y.append(y_test[i, 0])
    X1_test,  y_test = np.array(X_test1),  np.array(y)
    regressor1 = LSTM_model(38, X1_train, y_train,X1_test,  y_test, 32, 100)
    plt.plot(regressor1.history.history['loss'], label='train')
    plt.plot(regressor1.history.history['val_loss'], label='val')
    plt.legend()
    plt.show()

    #训练集
    y_train_pred= regressor1.predict(X1_train)
    #测试集
    y_test_pred= regressor1.predict(X1_test)

    print('=============训练集===============')
    print("The R2 score on the Train set is:\t{:0.8f}".format(r2_score(y_train, y_train_pred)))
    print("The MSE on the Train set is:\t{:0.8f}".format(mean_squared_error(y_train, y_train_pred)))
    print("The MAE on the Train set is:\t{:0.8f}".format(mean_absolute_error(y_train, y_train_pred)))
    print('=============测试集===============')
    print("The R2 score on the Test set is:\t{:0.8f}".format(r2_score(y_test, y_test_pred)))
    print("The MSE on the Test set is:\t{:0.8f}".format(mean_squared_error(y_test, y_test_pred)))
    print("The MAE on the Test set is:\t{:0.8f}".format(mean_absolute_error(y_test, y_test_pred)))




    #逆标准化之后的结果
    y_train = y_train.reshape(-1, 1)
    y_train = a.inverse_transform(y_train)
    y_train_pred = a.inverse_transform(y_train_pred)
    # print(y_train.shape)
    # print(y_train_pred.shape)
    plt.figure(figsize=(10, 4))
    plt.plot(y_train, color='red')
    plt.plot(y_train_pred, color='blue',
             label='Predicted Price By LSTM(inverse_scale)On Train dataset')
    plt.xlabel('Time')
    plt.ylabel('Price')
    plt.legend()
    plt.show()
    rmse1 = np.sqrt(mean_squared_error(y_train, y_train_pred))
    mape1 = mape(y_train ,y_train_pred)
    print('=============训练集===============')
    print("The R2 score on the Train set is:\t{:0.8f}".format(r2_score(y_train, y_train_pred)))
    print("The MSE on the Train set is:\t{:0.8f}".format(mean_squared_error(y_train, y_train_pred)))
    print("The RMSE on the Train set is:\t{:0.8f}".format(rmse1))
    print("The MAPE on the Train set is:\t{:0.8f}".format(mape1))
    print("The MAE on the Train set is:\t{:0.8f}".format(mean_absolute_error(y_train, y_train_pred)))

    y_test = y_test.reshape(-1, 1)
    y_test = b.inverse_transform(y_test)
    y_test_pred = b.inverse_transform(y_test_pred)
    plt.figure(figsize=(10, 4))
    plt.plot(y_test, color='red')
    plt.plot(y_test_pred, color='blue',
             label='Predicted Price By LSTM(inverse_scale)On Test dataset')
    plt.xlabel('Time')
    plt.ylabel('Price')
    plt.legend()
    plt.show()
    rmse2 = np.sqrt(mean_squared_error(y_test, y_test_pred))
    mape2 = mape(y_test ,y_test_pred)
    print('=============测试集===============')
    print("The R2 score on the Test set is:\t{:0.8f}".format(r2_score(y_test, y_test_pred)))
    print("The MSE on the Test set is:\t{:0.8f}".format(mean_squared_error(y_test, y_test_pred)))
    print("The RMSE on the Test set is:\t{:0.8f}".format(rmse2))
    print("The MAPE on the Train set is:\t{:0.8f}".format(mape2))
    print("The MAE on the Test set is:\t{:0.8f}".format(mean_absolute_error(y_test, y_test_pred)))

    y_var_test = y_test[1:] - y_test[:len(y_test) - 1]
    y_var_predict = y_test_pred[1:] - y_test_pred[:len(y_test_pred) - 1]
    txt = np.zeros(len(y_var_test))
    for i in range(len(y_var_test - 1)):
        txt[i] = np.sign(y_var_test[i]) == np.sign(y_var_predict[i])
    result = sum(txt) / len(txt)
    print('预测涨跌正确:', result)

    print(r2_score(y_train, y_train_pred))
    print(mean_squared_error(y_train, y_train_pred))
    print(rmse1)
    print(mape1)
    print(mean_absolute_error(y_train, y_train_pred))
    print(' ')
    print(r2_score(y_test, y_test_pred))
    print(mean_squared_error(y_test, y_test_pred))
    print(rmse2)
    print(mape2)
    print(mean_absolute_error(y_test, y_test_pred))
    print(result)

    # 1. 创建文件对象
    f = open('C:/Users/Big Data/Desktop/GLSTM.csv', 'w', encoding='utf-8')
    # 2. 基于文件对象构建 csv写入对象
    csv_writer = csv.writer(f)
    # 3. 构建列表头
    csv_writer.writerow(['y_test', 'GLSTM'])
    # 4. 写入csv文件内容
    y_test_m = y_test.flatten().tolist()
    y_test_pred_m = y_test_pred.flatten().tolist()
    csv_writer.writerow([y_test_m, y_test_pred_m])
    # 5. 关闭文件
    f.close()


import numpy as np
import skfuzzy as fuzz
import skfuzzy.control as ctrl
import pandas as pd
import warnings
import matplotlib.pyplot as plt
warnings.filterwarnings('ignore')



def run(path):
    """
    This function does everything in one shot. TODO: Break this function down.
    :return: None
    """
    wen1 = path.split('/')[-1]
    wen2 = wen1.split('.')[0]


    df= pd.read_csv(path, index_col=0)
    df =df.dropna()
    df = df.reset_index()

    df['L_upper'] = (df['High'] - np.maximum(df['Open'], df['Close'])) / df['Open']
    df['L_lower'] = (np.minimum(df['Open'], df['Close']) - df['Low']) / df['Open']
    df['L_body'] = (np.maximum(df['Open'], df['Close']) - np.minimum(df['Open'], df['Close'])) / df['Close']
    # 修正范围（0-100）
    df['L_upper'] = (df['L_upper'] - min(df['L_upper'])) / (max(df['L_upper']) - min(df['L_upper'])) * 100
    df['L_lower'] = (df['L_lower'] - min(df['L_lower'])) / (max(df['L_lower']) - min(df['L_lower'])) * 100
    df['L_body'] = (df['L_body'] - min(df['L_body'])) / (max(df['L_body']) - min(df['L_body'])) * 100

    return df,wen2


def fuzzification(df,wen2):
    # 创建模糊控制变量
    L_upper_range = np.arange(0, 101, 1, np.float64)
    L_lower_range = np.arange(0, 101, 1, np.float64)
    L_body_range = np.arange(0, 101, 1, np.float64)
    R_pos_range = np.arange(0, 101, 1, np.float64)
    R_size_range = np.arange(0, 101, 1, np.float64)
    # 创建模糊控制变量
    L_upper = ctrl.Antecedent(L_upper_range, 'L_upper')
    L_lower = ctrl.Antecedent(L_lower_range, 'L_lower')
    L_body = ctrl.Antecedent(L_body_range, 'L_body')
    R_pos = ctrl.Consequent(R_pos_range, 'R_pos')
    R_size = ctrl.Consequent(R_size_range, 'R_size')

    # 上影线隶属度函数
    L_upper['NULL'] = fuzz.trimf(L_upper_range, [0, 0, 5])

    L_upper['SHORT'] = fuzz.trapmf(L_upper_range, [0, 5, 25, 37.5])

    L_upper['MIDDLE'] = fuzz.trapmf(L_upper_range, [25, 37.5, 62.5, 75])

    L_upper['LONG'] = fuzz.trapmf(L_upper_range, [62.5, 75, 100, 100])

    # 下影线隶属度函数
    L_lower['NULL'] = fuzz.trimf(L_lower_range, [0, 0, 5])

    L_lower['SHORT'] = fuzz.trapmf(L_lower_range, [0, 5, 25, 37.5])

    L_lower['MIDDLE'] = fuzz.trapmf(L_lower_range, [25, 37.5, 62.5, 75])

    L_lower['LONG'] = fuzz.trapmf(L_lower_range, [62.5, 75, 100, 100])

    # 实体隶属度函数
    L_body['NULL'] = fuzz.trimf(L_body_range, [0, 0, 5])

    L_body['SHORT'] = fuzz.trapmf(L_body_range, [0, 5, 25, 37.5])

    L_body['MIDDLE'] = fuzz.trapmf(L_body_range, [25, 37.5, 62.5, 75])

    L_body['LONG'] = fuzz.trapmf(L_body_range, [62.5, 75, 100, 100])

    # R_pos隶属度函数
    R_pos['DOWN'] = fuzz.trimf(R_pos_range, [0, 0, 25])

    R_pos['CENTER_DOWN'] = fuzz.trimf(R_pos_range, [0, 25, 50])

    R_pos['CENTER'] = fuzz.trimf(R_pos_range, [25, 50, 75])

    R_pos['CENTER_UP'] = fuzz.trimf(R_pos_range, [50, 75, 100])

    R_pos['UP'] = fuzz.trimf(R_pos_range, [75, 100, 100])

    # R_size隶属度函数
    R_size['LOW'] = fuzz.trimf(R_size_range, [0, 0, 25])

    R_size['MEDIUM_LOW'] = fuzz.trimf(R_size_range, [0, 25, 50])

    R_size['MEDIUM'] = fuzz.trimf(R_size_range, [25, 50, 75])

    R_size['MEDIUM_EQUAL'] = fuzz.trimf(R_size_range, [50, 75, 100])

    R_size['EQUAL'] = fuzz.trimf(R_size_range, [75, 100, 100])

    # 设定输出powder的解模糊方法——质心解模糊方式

    R_pos.defuzzify_method = 'centroid'
    R_size.defuzzify_method = 'centroid'

    # 建立模糊规则
    rule1 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['LONG'] & L_body['LONG']),
                      consequent=(R_size['MEDIUM_EQUAL'], R_pos['CENTER']), label='rule 1')
    rule2 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['LONG'] & L_body['MIDDLE']),
                      consequent=(R_size['MEDIUM'], R_pos['CENTER']), label='rule 2')
    rule3 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['LONG'] & L_body['SHORT']),
                      consequent=(R_size['MEDIUM_LOW'], R_pos['CENTER']), label='rule 3')
    rule4 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['LONG'] & L_body['NULL']),
                      consequent=(R_size['LOW'], R_pos['CENTER']), label='rule 4')
    #############################################
    rule5 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['MIDDLE'] & L_body['LONG']),
                      consequent=(R_size['MEDIUM'], R_pos['CENTER_DOWN']), label='rule 5')
    rule6 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['MIDDLE'] & L_body['MIDDLE']),
                      consequent=(R_size['MEDIUM_LOW'], R_pos['CENTER_DOWN']), label='rule 6')
    rule7 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['MIDDLE'] & L_body['SHORT']),
                      consequent=(R_size['LOW'], R_pos['CENTER_DOWN']), label='rule 7')
    rule8 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['MIDDLE'] & L_body['NULL']),
                      consequent=(R_size['LOW'], R_pos['CENTER_DOWN']), label='rule 8')
    ################################################
    rule9 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['SHORT'] & L_body['LONG']),
                      consequent=(R_size['MEDIUM_EQUAL'], R_pos['DOWN']), label='rule 9')
    rule10 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['SHORT'] & L_body['MIDDLE']),
                       consequent=(R_size['MEDIUM'], R_pos['DOWN']), label='rule 10')
    rule11 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['SHORT'] & L_body['SHORT']),
                       consequent=(R_size['MEDIUM_LOW'], R_pos['DOWN']), label='rule 11')
    rule12 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['SHORT'] & L_body['NULL']),
                       consequent=(R_size['LOW'], R_pos['DOWN']), label='rule 12')
    ##############################################
    rule13 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['NULL'] & L_body['LONG']),
                       consequent=(R_size['MEDIUM_EQUAL'], R_pos['DOWN']), label='rule 13')
    rule14 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['NULL'] & L_body['MIDDLE']),
                       consequent=(R_size['MEDIUM'], R_pos['DOWN']), label='rule 14')
    rule15 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['NULL'] & L_body['SHORT']),
                       consequent=(R_size['MEDIUM_LOW'], R_pos['DOWN']), label='rule 15')
    rule16 = ctrl.Rule(antecedent=(L_upper['LONG'] & L_lower['NULL'] & L_body['NULL']),
                       consequent=(R_size['LOW'], R_pos['DOWN']), label='rule 16')
    #################################################
    rule17 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['LONG'] & L_body['LONG']),
                       consequent=(R_size['MEDIUM'], R_pos['CENTER_UP']), label='rule 17')
    rule18 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['LONG'] & L_body['MIDDLE']),
                       consequent=(R_size['MEDIUM_LOW'], R_pos['CENTER_UP']), label='rule 18')
    rule19 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['LONG'] & L_body['SHORT']),
                       consequent=(R_size['LOW'], R_pos['CENTER_UP']), label='rule 19')
    rule20 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['LONG'] & L_body['NULL']),
                       consequent=(R_size['LOW'], R_pos['CENTER_UP']), label='rule 20')
    #################################################
    rule21 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['MIDDLE'] & L_body['LONG']),
                       consequent=(R_size['MEDIUM_EQUAL'], R_pos['CENTER']), label='rule 21')
    rule22 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['MIDDLE'] & L_body['MIDDLE']),
                       consequent=(R_size['MEDIUM'], R_pos['CENTER']), label='rule 22')
    rule23 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['MIDDLE'] & L_body['SHORT']),
                       consequent=(R_size['MEDIUM_LOW'], R_pos['CENTER']), label='rule 23')
    rule24 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['MIDDLE'] & L_body['NULL']),
                       consequent=(R_size['LOW'], R_pos['CENTER']), label='rule 24')
    #################################################
    rule25 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['SHORT'] & L_body['LONG']),
                       consequent=(R_size['MEDIUM_EQUAL'], R_pos['CENTER_DOWN']), label='rule 25')
    rule26 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['SHORT'] & L_body['MIDDLE']),
                       consequent=(R_size['MEDIUM'], R_pos['CENTER_DOWN']), label='rule 26')
    rule27 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['SHORT'] & L_body['SHORT']),
                       consequent=(R_size['MEDIUM_LOW'], R_pos['CENTER_DOWN']), label='rule 27')
    rule28 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['SHORT'] & L_body['NULL']),
                       consequent=(R_size['LOW'], R_pos['CENTER_DOWN']), label='rule 28')
    #################################################
    rule29 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['NULL'] & L_body['LONG']),
                       consequent=(R_size['EQUAL'], R_pos['DOWN']), label='rule 29')
    rule30 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['NULL'] & L_body['MIDDLE']),
                       consequent=(R_size['MEDIUM_EQUAL'], R_pos['DOWN']), label='rule 30')
    rule31 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['NULL'] & L_body['SHORT']),
                       consequent=(R_size['MEDIUM_LOW'], R_pos['DOWN']), label='rule 31')
    rule32 = ctrl.Rule(antecedent=(L_upper['MIDDLE'] & L_lower['NULL'] & L_body['NULL']),
                       consequent=(R_size['LOW'], R_pos['DOWN']), label='rule 32')
    ###################################################
    rule33 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['LONG'] & L_body['LONG']),
                       consequent=(R_size['MEDIUM_EQUAL'], R_pos['UP']), label='rule 33')
    rule34 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['LONG'] & L_body['MIDDLE']),
                       consequent=(R_size['MEDIUM'], R_pos['UP']), label='rule 34')
    rule35 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['LONG'] & L_body['SHORT']),
                       consequent=(R_size['MEDIUM_LOW'], R_pos['UP']), label='rule 35')
    rule36 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['LONG'] & L_body['NULL']),
                       consequent=(R_size['LOW'], R_pos['UP']), label='rule 36')
    #############################################
    rule37 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['MIDDLE'] & L_body['LONG']),
                       consequent=(R_size['MEDIUM_EQUAL'], R_pos['CENTER_UP']), label='rule 37')
    rule38 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['MIDDLE'] & L_body['MIDDLE']),
                       consequent=(R_size['MEDIUM'], R_pos['CENTER_UP']), label='rule 38')
    rule39 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['MIDDLE'] & L_body['SHORT']),
                       consequent=(R_size['MEDIUM_LOW'], R_pos['CENTER_UP']), label='rule 39')
    rule40 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['MIDDLE'] & L_body['NULL']),
                       consequent=(R_size['LOW'], R_pos['CENTER_UP']), label='rule 40')
    ################################################
    rule41 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['SHORT'] & L_body['LONG']),
                       consequent=(R_size['EQUAL'], R_pos['CENTER']), label='rule 41')
    rule42 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['SHORT'] & L_body['MIDDLE']),
                       consequent=(R_size['MEDIUM_EQUAL'], R_pos['CENTER']), label='rule 42')
    rule43 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['SHORT'] & L_body['SHORT']),
                       consequent=(R_size['MEDIUM'], R_pos['CENTER']), label='rule 43')
    rule44 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['SHORT'] & L_body['NULL']),
                       consequent=(R_size['LOW'], R_pos['CENTER']), label='rule 44')
    ##############################################
    rule45 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['NULL'] & L_body['LONG']),
                       consequent=(R_size['EQUAL'], R_pos['CENTER_DOWN']), label='rule 45')
    rule46 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['NULL'] & L_body['MIDDLE']),
                       consequent=(R_size['EQUAL'], R_pos['CENTER_DOWN']), label='rule 46')
    rule47 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['NULL'] & L_body['SHORT']),
                       consequent=(R_size['MEDIUM_EQUAL'], R_pos['CENTER_DOWN']), label='rule 47')
    rule48 = ctrl.Rule(antecedent=(L_upper['SHORT'] & L_lower['NULL'] & L_body['NULL']),
                       consequent=(R_size['LOW'], R_pos['CENTER_DOWN']), label='rule 48')
    #################################################
    rule49 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['LONG'] & L_body['LONG']),
                       consequent=(R_size['MEDIUM_EQUAL'], R_pos['UP']), label='rule 49')
    rule50 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['LONG'] & L_body['MIDDLE']),
                       consequent=(R_size['MEDIUM'], R_pos['UP']), label='rule 50')
    rule51 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['LONG'] & L_body['SHORT']),
                       consequent=(R_size['MEDIUM_LOW'], R_pos['UP']), label='rule 51')
    rule52 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['LONG'] & L_body['NULL']),
                       consequent=(R_size['LOW'], R_pos['UP']), label='rule 52')
    #################################################
    rule53 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['MIDDLE'] & L_body['LONG']),
                       consequent=(R_size['EQUAL'], R_pos['UP']), label='rule 53')
    rule54 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['MIDDLE'] & L_body['MIDDLE']),
                       consequent=(R_size['MEDIUM_EQUAL'], R_pos['UP']), label='rule 54')
    rule55 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['MIDDLE'] & L_body['SHORT']),
                       consequent=(R_size['MEDIUM_LOW'], R_pos['UP']), label='rule 55')
    rule56 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['MIDDLE'] & L_body['NULL']),
                       consequent=(R_size['LOW'], R_pos['UP']), label='rule 56')
    #################################################
    rule57 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['SHORT'] & L_body['LONG']),
                       consequent=(R_size['EQUAL'], R_pos['CENTER_UP']), label='rule 57')
    rule58 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['SHORT'] & L_body['MIDDLE']),
                       consequent=(R_size['EQUAL'], R_pos['CENTER_UP']), label='rule 58')
    rule59 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['SHORT'] & L_body['SHORT']),
                       consequent=(R_size['MEDIUM_EQUAL'], R_pos['CENTER_UP']), label='rule 59')
    rule60 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['SHORT'] & L_body['NULL']),
                       consequent=(R_size['MEDIUM'], R_pos['CENTER_UP']), label='rule 60')
    #################################################
    rule61 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['NULL'] & L_body['LONG']),
                       consequent=(R_size['EQUAL'], R_pos['CENTER']), label='rule 61')
    rule62 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['NULL'] & L_body['MIDDLE']),
                       consequent=(R_size['EQUAL'], R_pos['CENTER']), label='rule 62')
    rule63 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['NULL'] & L_body['SHORT']),
                       consequent=(R_size['MEDIUM_EQUAL'], R_pos['CENTER']), label='rule 63')
    rule64 = ctrl.Rule(antecedent=(L_upper['NULL'] & L_lower['NULL'] & L_body['NULL']),
                       consequent=(R_size['MEDIUM'], R_pos['CENTER']), label='rule 64')

    # 隶属度函数可视化

    L_upper.view()
    L_lower.view()
    L_body.view()
    R_pos.view()
    R_size.view()
    plt.show()

    # 系统和运行环境初始化
    system = ctrl.ControlSystem(rules=[rule1, rule2, rule3, rule4, rule5, rule6, rule7, rule8, rule9, rule10,
                                       rule11, rule12, rule13, rule14, rule15, rule16, rule17, rule18, rule19,
                                       rule20,
                                       rule21, rule22, rule23, rule24, rule25, rule26, rule27, rule28, rule29,
                                       rule30,
                                       rule31, rule32, rule33, rule34, rule35, rule36, rule37, rule38, rule39,
                                       rule40,
                                       rule41, rule42, rule43, rule44, rule45, rule46, rule47, rule48, rule49,
                                       rule50,
                                       rule51, rule52, rule53, rule54, rule55, rule56, rule57, rule58, rule59,
                                       rule60,
                                       rule61, rule62, rule63, rule64])

    sim = ctrl.ControlSystemSimulation(system)

    # 得到模糊输出
    df['Rpos'] = np.zeros(shape=(df.shape[0], 1))
    df['Rsize'] = np.zeros(shape=(df.shape[0], 1))
    for i in range(0, df.shape[0]):
        sim.input['L_upper'] = df['L_upper'][i]
        sim.input['L_lower'] = df['L_lower'][i]
        sim.input['L_body'] = df['L_body'][i]
        sim.compute()  # 运行系统
        df['Rpos'][i] = sim.output['R_pos']
        df['Rsize'][i] = sim.output['R_size']
    df.to_csv('C:/Users/Big Data/Desktop/output/{}.csv'.format(wen2))
    return df

if __name__ == '__main__':
    path='C:/Users/Big Data/Desktop/output/原始数据/WTI.csv'
    df,wen2=run(path)
    df=fuzzification(df,wen2)


## 2. GA优化算法
##优化神经元，迭代次数和批次大小

import random
import math
import numpy as np
import pandas as pd
from LSTM import LSTM_model
from LSTM import data_split
import matplotlib.pyplot as plt
from sklearn.preprocessing import MinMaxScaler
from sklearn.metrics import mean_squared_error

class GA(object):
    ###2.1 初始化
    def __init__(self, population_size, chromosome_num, chromosome_length, max_value,min_value, iter_num, pc, pm,x_train,y_train):
        '''初始化参数
        input:population_size(int):种群数
              chromosome_num(int):染色体数，对应需要寻优的参数个数
              chromosome_length(int):染色体的基因长度
              max_value(float):作用于二进制基因转化为染色体十进制数值
              iter_num(int):迭代次数
              pc(float):交叉概率阈值(0<pc<1)
              pm(float):变异概率阈值(0<pm<1)
        '''
        self.population_size = population_size
        self.chromosome_length = chromosome_length
        self.chromosome_num = chromosome_num
        self.iter_num = iter_num
        self.max_value = max_value
        self.pc = pc  ##一般取值0.4~0.99
        self.pm = pm  ##一般取值0.0001~0.1
        self.x_train=x_train
        self.y_train = y_train
        self.min_value=min_value

    def species_origin(self):
        '''初始化种群、染色体、基因
        input:self(object):定义的类参数
        output:population(list):种群
        '''
        population = []
        ## 分别初始化四个染色体
        for i in range(chromosome_num):
            tmp1 = []  ##暂存器1，用于暂存一个染色体的全部可能二进制基因取值
            for j in range(population_size):
                tmp2 = []  ##暂存器2，用于暂存一个染色体的基因的每一位二进制取值
                for l in range(chromosome_length[i]):
                    tmp2.append(random.randint(0, 1))
                tmp1.append(tmp2)
            population.append(tmp1)
        return population

    ###2.2 计算适应度函数值
    def translation(self, population):
        '''将染色体的二进制基因转换为十进制取值
        input:self(object):定义的类参数
              population(list):种群
        output:population_decimalism(list):种群每个染色体取值的十进制数
        '''
        population_decimalism = []
        for i in range(len(population)):
            tmp = []  ##暂存器，用于暂存一个染色体的全部可能十进制取值
            for j in range(len(population[0])):
                total = 0
                for l in range(len(population[i][j])):
                    total += population[i][j][l] * (math.pow(2, l))
                tmp.append(total)
            population_decimalism.append(tmp)
        return population_decimalism

    def fitness(self, population):
        '''计算每一组染色体对应的适应度函数值(解码)
        input:self(object):定义的类参数
              population(list):种群
        output:fitness_value(list):每一组染色体对应的适应度函数值
        '''
        fitness = []
        population_decimalism = self.translation(population)
        for i in range(len(population[0])):
            tmp = []  ##暂存器，用于暂存每组染色体十进制数值
            for j in range(len(population)): #解码
                value =self.min_value[j] + (population_decimalism[j][i] * (self.max_value[j] - self.min_value[j]) / (math.pow(2, self.chromosome_length[j]) - 1))
                tmp.append(int(value))


            ## loss 为适应度函数值
            lstm_unit = int(tmp[1])
            # num(0)为神经元
            seq_len =int(tmp[0])
            # 验证模型
            regressor1 = LSTM_model(lstm_unit, self.x_train, self.y_train,seq_len,10, 500)#
            X_test1, y = [], []
            for i in range(seq_len, self.x_train.shape[0]):
                X_test1.append(self.x_train[i - seq_len:i, :])
                y.append(self.y_train[i, 0])
            x_test, y_test = np.array(X_test1), np.array(y)
            y_test_pred = regressor1.predict(x_test)
            y_test = y_test.reshape(-1, 1)
            loss= np.sqrt(mean_squared_error(y_test, y_test_pred))
            fitness.append(loss)
        ##将适应度函数值中为负数的数值排除
        fitness_value = []
        num = len(fitness)
        for l in range(num):
            if (fitness[l] > 0):
                tmp1 = fitness[l]
            else:
                tmp1 = 0.0
            fitness_value.append(tmp1)
        return fitness_value

    ###2.3 选择操作
    def sum_value(self, fitness_value):
        '''适应度求和
        input:self(object):定义的类参数
              fitness_value(list):每组染色体对应的适应度函数值
        output:total(float):适应度函数值之和
        '''
        total = 0.0
        for i in range(len(fitness_value)):
            total += fitness_value[i]
        return total

    def cumsum(self, fitness1):
        '''计算适应度函数值累加列表
        input:self(object):定义的类参数
              fitness1(list):适应度函数值列表
        output:适应度函数值累加列表
        '''
        ##计算适应度函数值累加列表
        for i in range(len(fitness1) - 1, -1, -1):  # range(start,stop,[step]) # 倒计数
            total = 0.0
            j = 0
            while (j <= i):
                total += fitness1[j]
                j += 1
            fitness1[i] = total

    def selection(self, population, fitness_value):
        '''选择操作
        input:self(object):定义的类参数
              population(list):当前种群
              fitness_value(list):每一组染色体对应的适应度函数值
        '''
        new_fitness = []  ## 用于存储适应度函归一化数值
        total_fitness = self.sum_value(fitness_value)  ## 适应度函数值之和
        for i in range(len(fitness_value)):
            new_fitness.append(fitness_value[i] / total_fitness)
        self.cumsum(new_fitness)

        ms = []  ##用于存档随机数
        pop_len = len(population[0])  ##种群数

        for i in range(pop_len):
            ms.append(random.randint(0, 1))
        ms.sort()  ## 随机数从小到大排列

        ##存储每个染色体的取值指针
        fitin = 0
        newin = 0

        new_population = population

        ## 轮盘赌方式选择染色体
        while newin < pop_len & fitin < pop_len:
            if (ms[newin] < new_fitness[fitin]):
                for j in range(len(population)):
                    new_population[j][newin] = population[j][fitin]
                newin += 1
            else:
                fitin += 1
        population = new_population

    ### 2.4 交叉操作
    def crossover(self, population):
        '''交叉操作
        input:self(object):定义的类参数
              population(list):当前种群
        '''
        pop_len = len(population[0])

        for i in range(len(population)):
            for j in range(pop_len - 1):
                if (random.random() < self.pc):
                    cpoint = random.randint(0, len(population[i][j]))  ## 随机选择基因中的交叉点
                    ###实现相邻的染色体基因取值的交叉
                    tmp1 = []
                    tmp2 = []
                    # 将tmp1作为暂存器，暂时存放第i个染色体第j个取值中的前0到cpoint个基因，
                    # 然后再把第i个染色体第j+1个取值中的后面的基因，补充到tem1后面
                    tmp1.extend(population[i][j][0:cpoint])
                    tmp1.extend(population[i][j + 1][cpoint:len(population[i][j])])
                    # 将tmp2作为暂存器，暂时存放第i个染色体第j+1个取值中的前0到cpoint个基因，
                    # 然后再把第i个染色体第j个取值中的后面的基因，补充到tem2后面
                    tmp2.extend(population[i][j + 1][0:cpoint])
                    tmp2.extend(population[i][j][cpoint:len(population[i][j])])
                    # 将交叉后的染色体取值放入新的种群中
                    population[i][j] = tmp1
                    population[i][j + 1] = tmp2

    ### 2.5 变异操作
    def mutation(self, population):
        '''变异操作
        input:self(object):定义的类参数
              population(list):当前种群
        '''
        pop_len = len(population[0])  # 种群数
        Gene_len = len(population[0][0])  # 基因长度
        for i in range(len(population)):
            for j in range(pop_len):
                if (random.random() < self.pm):
                    mpoint = random.randint(0, Gene_len - 1)  ##基因变异位点
                    ##将第mpoint个基因点随机变异，变为0或者1
                    if (population[i][j][mpoint] == 1):
                        population[i][j][mpoint] = 0
                    else:
                        population[i][j][mpoint] = 1

    ### 2.6 找出当前种群中最好的适应度和对应的参数值
    def best(self, population_decimalism, fitness_value):
        '''找出最好的适应度和对应的参数值
        input:self(object):定义的类参数
              population(list):当前种群
              fitness_value:当前适应度函数值列表
        output:[bestparameters,bestfitness]:最优参数和最优适应度函数值
        '''
        pop_len = len(population_decimalism[0])
        bestparameters = []  ##用于存储当前种群最优适应度函数值对应的参数
        bestfitness = 10000.0  ##用于存储当前种群最优适应度函数值

        for i in range(0, pop_len):
            tmp = []
            if (fitness_value[i] < bestfitness):
                bestfitness = fitness_value[i]
                for j in range(len(population_decimalism)):
                    tmp.append(int(self.min_value[j] + (population_decimalism[j][i] * (self.max_value[j] - self.min_value[j]) / (math.pow(2, self.chromosome_length[j]) - 1))))
                    bestparameters = tmp

        return bestparameters, bestfitness

    ### 2.7 画出适应度函数值变化图
    def plot(self, results):
        '''画图
        '''
        X = []
        Y = []
        for i in range(self.iter_num):
            X.append(i + 1)
            Y.append(results[i])
        plt.plot(X, Y)
        plt.xlabel('Number of iteration', size=15)
        plt.ylabel('Value of RMSE', size=15)
        plt.title('GA_LSTM parameter optimization')
        plt.show()

    ### 2.8 主函数
    def main(self):
        results = []
        parameters = []
        best_fitness = 10000.0
        best_parameters = []
        ## 初始化种群
        population = self.species_origin()
        ## 迭代参数寻优
        for i in range(self.iter_num):
            ##计算适应函数数值列表
            fitness_value = self.fitness(population)
            ## 计算当前种群每个染色体的10进制取值
            population_decimalism = self.translation(population)
            ## 寻找当前种群最好的参数值和最优适应度函数值
            current_parameters, current_fitness = self.best(population_decimalism, fitness_value)
            print('current_parameters:', current_parameters, ';current_fitness:', current_fitness)
            ## 与之前的最优适应度函数值比较，如果更优秀则替换最优适应度函数值和对应的参数
            if current_fitness < best_fitness:
                best_fitness = current_fitness
                best_parameters = current_parameters
            print('iteration is :', i, ';Best parameters:', best_parameters, ';Best fitness', best_fitness)
            results.append(best_fitness)
            parameters.append(best_parameters)
            ## 种群更新
            ## 选择
            self.selection(population, fitness_value)
            ## 交叉
            self.crossover(population)
            ## 变异
            self.mutation(population)
        results.sort()
        self.plot(results)
        print('Final parameters are :', parameters[-1])
if __name__ == '__main__':
    print('----------------1.data process------------')
    data1 = pd.read_csv('C:/Users/C4S702/Desktop/output/fuzzy/IXIC.csv', parse_dates=['Date'], index_col=0,
                        encoding='utf-8')
    df = data1.reset_index()
    df = df.dropna()
    #seq_len = 30
    X1_train, y_train, X1_test, y_test, train_size, test_size, y = data_split(df)
    scaler = MinMaxScaler(feature_range=(0, 1))
    x_train = scaler.fit_transform(X1_train)
    y_train = scaler.fit_transform(y_train)
    x_test = scaler.fit_transform(X1_test)
    y_test = scaler.fit_transform(y_test)
    y_train=y_train.reshape(-1, 1)
    y_test = y_test.reshape(-1, 1)
    print('----------------2.GA_set-----------------')
    population_size = 10
    chromosome_num = 2
    max_value =[30, 100]
    min_value = [5, 32]
    chromosome_length = [5,7]
    iter_num = 20
    pc = 0.7
    pm = 0.2

    print('----------------3.GA_LSTM-----------------')
    ga = GA(population_size, chromosome_num, chromosome_length, max_value,min_value, iter_num, pc, pm, x_train, y_train)
    ga.main()



